//********************************************************************
//*** ResourcePool/Command.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Command.hh,v 1.7 2005/01/16 10:38:56 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_COMMAND_HH
#define RESOURCEPOOL_COMMAND_HH

#ifndef RESOURCEPOOL_EXECUTEFWD_HH
#include "ExecuteFwd.hh"
#endif

namespace fatalmind {

/**
 * The base class for all commands to be executed by ResourcePool.
 */
template<class PT>
class Command {
	protected:
		Command();
	public:
		/**
		 * The execute method is the one which actually carries out the work.
		 * This method gets called by ResourcePool with a (valid) plain
		 * resource which was obtained from the pool.
		 *
		 * On normal execution (no exception) the resource will be free()ed,
		 * on abnormal execution (exception) the resource will be failed.
		 * If you need to throw an functional exception which should not 
		 * cause failing of the resource, derive this exception from
		 * NoFailoverException.
		 */
		virtual void execute(typename PT::plainresource_t&) = 0;

	private:
		bool topLevel() const;
		void topLevel(bool tl);
		bool _tl;

	friend 
		class _RPExecute<PT>;
};

} // namespace fatalmind
#endif
