//********************************************************************
//*** ResourcePool/SQL/BindOut.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: BindOut.cc,v 1.7 2005/01/06 16:27:56 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "BindOut.hh"

namespace fatalmind {
namespace SQL {

template<class VBC, template<class> class CT>
template<typename T>
void BindOut<VBC, CT>::bindout(const unsigned int pos, T& val) {
	gc_ptr<VBC> ptr(new CT<T>(val));
	_outparam.resize(std::max<int>(pos + 1, _outparam.size()));
	_outparam[pos] = ptr;
}

template<class VBC, template<class> class CT>
bool BindOut<VBC, CT>::isnull(const unsigned int pos) const {
	if (pos <= _outparam.size()) {
		return (_outparam[pos]->isnull());
	} else {
		return false;
	}
}

template<class VBC, template<class> class CT>
VBC& BindOut<VBC, CT>::getOutParam(const unsigned int pos) {
	if (pos <= _outparam.size()) {
		return *(_outparam[pos]);
	} else {
		std::ostringstream os;
		os << "bindout(" << pos << ", ...) has not been called";
		throw SQLException(os.str(), "07001", "UKN", false);
	}
}

template<class VBC, template<class> class CT>
void BindOut<VBC, CT>::instantiations() {
	short int i1;
	bindout(0, i1);
	int i2;
	bindout(0, i2);
	long int i3;
	bindout(0, i3);
	unsigned short int i4;
	bindout(0, i4);
	unsigned int i5;
	bindout(0, i5);
	unsigned long int i6;
	bindout(0, i6);

	float t2;
	bindout<float>(0, t2);
	double t3;
	bindout<double>(0, t3);

	std::string t5;
	bindout<std::string>(0, t5);
}

} // namespace SQL
} // namespece fatalmind
