//********************************************************************
//*** ResourcePool/SQL/SQLSelectRow.t.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelectRow.t.cc,v 1.10 2005/01/06 16:27:56 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "t/main.hh"

using namespace std;
using namespace fatalmind;

#include <memory>
#include <iostream>

std::auto_ptr<RP> _rp;

void test_init()
{
	if (_rp.get() == 0) {
		_rp.reset(new RP(getFactory()));
	}
}

void test_SQLSelectRow_init()
{
	test_init();
	init(*_rp);

	RP::Command::SQLExecute init("insert into hirsch (id, ukn) values (?, ?)");
	init.bind<std::string>(1, "hohoho ' kasdfjs \" sdf");
	for (int i = 0; i < 600; ++i) {
		init.bind(0, i);
		_rp->execute(init);
	}
}


void test_SQLSelectRow_one()
{
	test_init();
	RP::Command::SQLSelectRow cmd("select ukn,id  from hirsch where id = ?");
	int col1;
	std::string col2;

	cmd.bind(0, 4);
	cmd.bindout(0, col2);
	cmd.bindout(1, col1);

	_rp->execute(cmd);

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
	assertEqual(col1, 4);
}

void test_perf_SQLSelectRow_one_spec_outer()
{
	test_init();
	RP::Command::SQLSelectRow cmd("select id, ukn from hirsch where id = ?");
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);
	for (int i = 0; i < 600; ++i) {
		cmd.bind(0, i);
		_rp->execute(cmd);

		assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
		assertEqual(col1, i);
	}
}

void test_perf_SQLSelectRow_one_spec_inner()
{
	test_init();
	int col1;
	std::string col2;

	for (int i = 0; i < 600; ++i) {
		RP::Command::SQLSelectRow cmd("select id, ukn from hirsch where id = ?");
		cmd.bindout(0, col1);
		cmd.bindout(1, col2);

		cmd.bind(0, i);
		_rp->execute(cmd);

		assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
		assertEqual(col1, i);
	}
}

// TODO: mysql?, others
void test_perf_SQLSelectRow_first_one()
{
	test_init();
	RP::Command::SQLSelectRow cmd("select id, ukn from hirsch where rownum <= 1");
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);
	for (int i = 0; i < 600; ++i) {
		_rp->execute(cmd);

		assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
	}
}
// TODO: check behaviour if not found or more then one found..

void test_SQLSelectRow_cleanup() 
{
	test_init();
    drop(*_rp);
	_rp.reset(0);
}
