//********************************************************************
//*** Thread/Synchronize.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Synchronize.hh,v 1.7 2005/01/06 16:28:06 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */
#ifndef INCLUDED_THREAD_SYNCHRONIZE_HH
#define INCLUDED_THREAD_SYNCHRONIZE_HH

namespace fatalmind {

template<class Lock>
class Synchronize {
	public:
		inline Synchronize(Lock& lock): _lock(lock) {
			_lock.lock();	
		}

		inline Synchronize(Synchronize& rho)
		: _lock(rho._lock)
		{
			_lock.recursive_ok();
		}

		inline ~Synchronize() {
			_lock.unlock();
		}
	private:
		Lock& _lock;
};

// NopLock specialization
template<> class Synchronize<NopLock<> > {
	public:
		inline Synchronize(NopLock<> lock) {
		}
		inline ~Synchronize() {
		}
};

/*
 * 2 way synchro
 */

template<class Lock, class Lock2>
class Synchronize2 {
	public:
		inline Synchronize2(Lock& lock, Lock2& lock2)
		: _lock(lock) 
		, _lock2(lock2)
		{
			_lock.lock();	
			// TODO:: order locking for deadlock prevention
			if (&_lock != &_lock) {
				_lock2.lock();
			}
		}

		inline ~Synchronize2() {
			if (&_lock != &_lock) {
				_lock2.unlock();
			}
			_lock.unlock();
		}
	private:
		Lock& _lock;
		Lock2& _lock2;
};

template<> class Synchronize2<NopLock<> , NopLock<> > {
	public:
		inline Synchronize2(NopLock<>& lock, NopLock<>& lock2) {
		}
		inline ~Synchronize2() {
		}
};

} //namespace fatalmind

#endif
